"""Note Class."""
from .models import SpeedCurveCore


class Note(SpeedCurveCore):
    """SpeedCurve Note Class."""

    def _update_attributes(self, json):
        self.id = json.get('note_id')
        self.site_id = json.get('site_id')
        self.note = json.get('note')
        self.detail = json.get('detail')

    def __repr__(self):
        """Return formatted string of instance."""
        return '<Note [{}]>'.format(self.id)



# -*- coding: utf-8 -*-
from .models import SpeedCurveCore


class Test(SpeedCurveCore):
    """The :class:`Test <Test>` object. """

    def _update_attributes(self, json):
        self.browser = json.get('browser')
        self.browser_version = json.get('browser_version')
        self.byte = json.get('byte')
        self.css_requests = json.get('css_requests')
        # TODO: change to datetime object
        self.day = json.get('day')
        self.flash_requests = json.get('flash_requests')
        self.font_requests = json.get('font_requests')
        self.font_size = json.get('font_size')
        self.har = json.get('har')
        self.html_requests = json.get('html_requests')
        self.html_size = json.get('html_size')
        self.image_requests = json.get('image_requests')
        self.image_saving = json.get('image_saving')
        self.image_size = json.get('image_size')
        self.js_requests = json.get('js_requests')
        self.js_size = json.get('js_size')
        self.loaded = json.get('loaded')
        self.other_requests = json.get('other_requests')
        self.other_size = json.get('other_size')
        self.pagespeed = json.get('pagespeed')
        self.render = json.get('render')
        self.requests = json.get('requests')
        self.run = json.get('run')
        self.screen = json.get('screen')
        self.size = json.get('size')
        self.speedindex = json.get('speedindex')
        self.test = json.get('test')
        self.text_requests = json.get('text_requests')
        self.timestamp = json.get('timestamp')
        self.timezone = json.get('timezone')

    def __repr__(self):
        return 'Test <speedcurve.tests.Test [{}]>'.format(self.test)



"""Url Class."""
from .models import SpeedCurveCore


class Url(SpeedCurveCore):
    """The :class:`Url <Url>` object."""

    def _update_attributes(self, url):
        self.url = url.get('url')
        self.tests = url.get('tests')

    def __repr__(self):
        return 'Url <speedcurve.urls.Url [{0}]'.format(self.url)



"""All exceptions for speedcurve library."""


class SpeedCurveError(Exception):
    """The base exception class."""

    def __init__(self, response):
        super(SpeedCurveError, self).__init__(response)
        self.response = response
        self.code = response.status_code
        self.errors = []

        try:
            error = response.json()
            self.errors = error.get('errors')
        except:
            self.msg = response.content or '[No Message]'


class AuthenticationFailed(SpeedCurveError):
    pass


class BadRequest(SpeedCurveError):
    pass


class ForbiddenError(SpeedCurveError):
    pass


class NotFoundError(SpeedCurveError):
    pass

error_classes = {
    400: BadRequest,
    401: AuthenticationFailed,
    403: ForbiddenError,
    404: NotFoundError
}


def get_error_for(response):
    """Return initialized exception."""
    status_code = response.status_code
    error_class = error_classes.get(status_code)
    return error_class(response)



from .models import SpeedCurveCore


class Deployment(SpeedCurveCore):
    """Deployment class."""

    def _update_attributes(self, json):
        self.id = json.get('deploy_id')
        self.tests_completed = json.get('tests-completed')
        self.status = json.get('status')
        self.tests_remaining = json.get('tests-remaining')
        self.note = json.get('note')
        self.detail = json.get('detail')

    def __repr__(self):
        return '<Deployment [{}]>'.format(self.id)



from json import dumps
from .session import SpeedCurveSession
from . import exceptions


class SpeedCurveObject(object):
    def __init__(self, json):
        self._update_attributes(json)

    def _update_attributes(self, json):
        pass


class SpeedCurveCore(SpeedCurveObject):

    def __init__(self, json, session=None, api_key=None):

        if hasattr(session, 'session'):
            session = session.session
        elif session is None:
            session = SpeedCurveSession(api_key=api_key)
        self.session = session
        self._as_json = json

        super(SpeedCurveCore, self).__init__(json)

    def as_json(self):
        return dumps(self._as_json)

    def _build_url(self, *args, **kwargs):
        return self.session.build_url(*args, **kwargs)

    def _get(self, url, *args, **kwargs):
        return self.session.get(url, *args, **kwargs)

    def _json(self, response, status_code):
        ret = None

        if self._expected_response(response, status_code, 404):
            ret = response.json()
        return ret

    def _expected_response(self, response, true_code, false_code):
        if response is not None:
            status_code = response.status_code
            if status_code == true_code:
                return True
            if true_code != false_code and status_code >= 400:
                raise exceptions.get_error_for(response)
        return False

    def _instance_or_null(self, instance_class=None, json=None):
        try:
            return instance_class(json, self)
        except TypeError:
            return instance_class(json)

    def _remove_none_values(self, dictionary):
        """Helper method to remove None values from dictionary."""
        data = dictionary.copy()
        if not data:
            return
        for key, value in data.items():
            if value is None:
                del(data[key])
        return data

    def _post(self, url, *args, **kwargs):
        return self.session.post(url, *args, **kwargs)



# -*- coding: utf-8 -*-
import requests


class SpeedCurveSession(requests.Session):

    def __init__(self, api_key=None):
        if api_key is None:
            raise ValueError("API Key Required")

        self.api_key = str(api_key)
        self.basic_authentication = (str(self.api_key), 'x')
        super(SpeedCurveSession, self).__init__()
        self.headers.update({
            'Content-Type': 'application/json'
        })
        self.base_url = 'https://api.speedcurve.com/v1'

    def build_url(self, *args, **kwargs):
        parts = [kwargs.get('base_url') or self.base_url]
        parts.extend(args)
        url = '/'.join(parts)
        return url

    def get(self, *args, **kwargs):
        kwargs['auth'] = self.basic_authentication
        return super(SpeedCurveSession, self).get(*args, **kwargs)

    def post(self, *args, **kwargs):
        kwargs['auth'] = self.basic_authentication
        return super(SpeedCurveSession, self).post(*args, **kwargs)



"""Speedcurve API."""

from .models import SpeedCurveCore
from .notes import Note
from .sites import Site
from .tests import Test
from .urls import Url
from .deployments import Deployment


class SpeedCurve(SpeedCurveCore):
    """Stored session information."""

    def __init__(self, api_key=None, session=None):
        """Constructor for SpeedCurve.

        :param string api_key: (optional) API key for authentication
        :returns: :class:`SpeedCurve <speedcurve.SpeedCurve>`
        """
        super(SpeedCurve, self).__init__({}, api_key=api_key, session=session)

    def add_deployment(self, site_id=None, note=None, detail=None):
        """Add a deployment and trigger round of testing.

        :param int site_id: (optional) site id to trigger deploy.
        :param string note: (required) short note used on site
        :param string detail: (optional) detail to display for more context
        :returns: :class:`Deployment <speedcurve.deployments.Deployment>`
        """
        data = {
            'site_id': site_id or None,
            'note': str(note) or '',
            'detail': str(detail) or ''
        }

        data = self._remove_none_values(data)

        headers = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        url = self._build_url('deploy')
        json = self._json(self._post(url, data=data, headers=headers), 200)
        if json:
            return self._instance_or_null(Deployment, json)

    def get_latest_deployment(self):
        """Retrieve latest deployment.

        :returns: :class:`Deployment <speedcurve.deployments.Deployment>`
        """
        url = self._build_url('deploy', 'latest')
        json = self._json(self._get(url), 200)
        if json:
            return self._instance_or_null(Deployment, json)

    def get_deployment(self, id=None):
        """Retrieve a deployment specified by id.

        :params int id: (required) id of deployment
        :returns: :class:`Deployment <speedcurve.deployments.Deployment>`
        """
        url = self._build_url('deploy', str(id))
        json = self._json(self._get(url), 200)
        if json:
            return self._instance_or_null(Deployment, json)

    def notes(self):
        """Retrieve all notes for main site in User' account.

        :returns: Generator of :class:`Note <speedcurve.notes.Note>`
        """
        url = self._build_url('notes')
        json = self._json(self._get(url), 200)
        if json:
            notes = [
                self._instance_or_null(Note, note) for note in json['notes']
            ]
            return notes

    def sites(self):
        """Retrieve all sites for account."""
        url = self._build_url('sites')
        json = self._json(self._get(url), 200)
        sites = None
        if json:
            sites = [
                self._instance_or_null(Site, s) for s in json.get('sites')
            ]
        return sites

    def test(self, id=None):
        """Retrieve test specified by test id.

        :param string id: (required) ID of test
        :returns: instance of :class:`Test <speedcurve.tests.Test>`
        """
        url = self._build_url('tests', str(id))
        json = self._json(self._get(url), 200)
        if json:
            return self._instance_or_null(Test, json)

    def url(self, id=None, days=30, browser='all'):
        """Retrieve url specified by id.

        :param int id: (required) id of URL
        :param int days: (optional) number of days of tests (max: 365)
        :param string browser: (optional) all, chrome, firefox, ie, or safari
        :returns: :class:`Url <speedcurve.urls.Url>`

        """
        url = self._build_url('urls', str(id))
        params = {
            'days': days,
            'browser': browser
        }
        json = self._json(self._get(url, params=params), 200)
        if json:
            return self._instance_or_null(Url, json)



from setuptools import setup

packages = [
    'speedcurve',
]

setup(
    author='Matt Chung',
    author_email='itsmemattchung@gmail.com',
    description=('Python wrapper for Speedcurve API'
                 '(https://api.speedcurve.com/)'),
    install_requires=[
        'requests'
    ],
    name='speedcurve.py',
    url='https://speedcurvepy.readthedocs.org',
    packages=packages,
    version='0.1.9'
)



# -*- coding: utf-8 -*-
from .models import SpeedCurveCore


class Site(SpeedCurveCore):
    """The :class:`Site <Site>` object. """

    def _update_attributes(self, site):
        self.id = site.get('site_id')
        self.name = site.get('name')
        self.median = site.get('median')
        self.urls = site.get('urls')

    def __repr__(self):
        return '<Site [{0}]>'.format(self.name)



# -*- coding: utf-8 -*-
from .speedcurve import SpeedCurve

__author__ = 'Matt Chung'
__copyright__ = 'Copyright 2014 Matt Chung'
__license__ = 'MIT'
__title__ = 'speedcurve'
__version__ = '0.1.0'

__all__ = (
    'SpeedCurve',
)
